#ifndef _LINUX_PID_H
#define _LINUX_PID_H

#define VPID_BIT	10
#define VPID_DIV	(1<<VPID_BIT)

#ifdef CONFIG_VE
#define __is_virtual_pid(pid)	((pid) & VPID_DIV)
#define is_virtual_pid(pid)	\
   (__is_virtual_pid(pid) || ((pid)==1 && !ve_is_super(get_exec_env())))
#else
#define __is_virtual_pid(pid)	0
#define is_virtual_pid(pid)	0
#endif

enum pid_type
{
	PIDTYPE_PID,
	PIDTYPE_TGID,
	PIDTYPE_PGID,
	PIDTYPE_SID,
	PIDTYPE_MAX
};

struct pid
{
	/* Try to keep pid_chain in the same cacheline as nr for find_pid */
	int nr;
	struct hlist_node pid_chain;
#ifdef CONFIG_VE
	int vnr;
#endif
	/* list of pids with the same nr, only one of them is in the hash */
	struct list_head pid_list;
};

#define pid_task(elem, type) \
	list_entry(elem, struct task_struct, pids[type].pid_list)

/*
 * attach_pid() and detach_pid() must be called with the tasklist_lock
 * write-held.
 */
extern int FASTCALL(attach_pid(struct task_struct *task, enum pid_type type, int nr));

extern void FASTCALL(detach_pid(struct task_struct *task, enum pid_type));

/*
 * look up a PID in the hash table. Must be called with the tasklist_lock
 * held.
 */
extern struct pid *FASTCALL(find_pid(enum pid_type, int));

extern int alloc_pidmap(void);
extern void FASTCALL(free_pidmap(int));
extern void switch_exec_pids(struct task_struct *leader, struct task_struct *thread);

#ifndef CONFIG_VE

#define vpid_to_pid(pid)	(pid)
#define __vpid_to_pid(pid)	(pid)
#define pid_type_to_vpid(pid, type)	(pid)
#define __pid_type_to_vpid(pid, type)	(pid)

#define comb_vpid_to_pid(pid)	(pid)
#define comb_pid_to_vpid(pid)	(pid)

#else

struct ve_struct;
extern void free_vpid(int vpid, struct ve_struct *ve);
extern int alloc_vpid(int pid, int vpid);
extern int vpid_to_pid(int pid);
extern int __vpid_to_pid(int pid);
extern pid_t pid_type_to_vpid(int type, pid_t pid);
extern pid_t _pid_type_to_vpid(int type, pid_t pid);

static inline int comb_vpid_to_pid(int vpid)
{
	int pid = vpid;

	if (vpid > 0) {
		pid = vpid_to_pid(vpid);
		if (unlikely(pid < 0))
			return 0;
	} else if (vpid < 0) {
		pid = vpid_to_pid(-vpid);
		if (unlikely(pid < 0))
			return 0;
		pid = -pid;
	}
	return pid;
}

static inline int comb_pid_to_vpid(int pid)
{
	int vpid = pid;

	if (pid > 0) {
		vpid = pid_type_to_vpid(PIDTYPE_PID, pid);
		if (unlikely(vpid < 0))
			return 0;
	} else if (pid < 0) {
		vpid = pid_type_to_vpid(PIDTYPE_PGID, -pid);
		if (unlikely(vpid < 0))
			return 0;
		vpid = -vpid;
	}
	return vpid;
}
#endif

#define do_each_task_pid_all(who, type, task)				\
	if ((task = find_task_by_pid_type_all(type, who))) {		\
		prefetch((task)->pids[type].pid_list.next);		\
		do {

#define while_each_task_pid_all(who, type, task)			\
		} while (task = pid_task((task)->pids[type].pid_list.next,\
						type),			\
			prefetch((task)->pids[type].pid_list.next),	\
			hlist_unhashed(&(task)->pids[type].pid_chain));	\
	}								\

#ifndef CONFIG_VE
#define __do_each_task_pid_ve(who, type, task, owner)			\
		do_each_task_pid_all(who, type, task)
#define __while_each_task_pid_ve(who, type, task, owner)		\
		while_each_task_pid_all(who, type, task)
#else /* CONFIG_VE */
#define __do_each_task_pid_ve(who, type, task, owner)			\
		do_each_task_pid_all(who, type, task)			\
			if (ve_accessible(VE_TASK_INFO(task)->owner_env, owner))
#define __while_each_task_pid_ve(who, type, task, owner)		\
		while_each_task_pid_all(who, type, task)
#endif /* CONFIG_VE */

#define do_each_task_pid_ve(who, type, task)				\
		__do_each_task_pid_ve(who, type, task, get_exec_env());
#define while_each_task_pid_ve(who, type, task)				\
		__while_each_task_pid_ve(who, type, task, get_exec_env());

extern int pid_alive(struct task_struct *p);

#endif /* _LINUX_PID_H */
