/*
 *  include/linux/ve_proto.h
 *
 *  Copyright (C) 2005  SWsoft
 *  All rights reserved.
 *  
 *  Licensing governed by "linux/COPYING.SWsoft" file.
 *
 */

#ifndef __VE_H__
#define __VE_H__

#ifdef CONFIG_VE

extern struct semaphore ve_call_guard;
extern rwlock_t ve_call_lock;

#ifdef CONFIG_SYSVIPC
extern void prepare_ipc(void);
extern int init_ve_ipc(struct ve_struct *);
extern void fini_ve_ipc(struct ve_struct *);
extern void ve_ipc_cleanup(void);
#endif

extern struct tty_driver *get_pty_driver(void);
extern struct tty_driver *get_pty_slave_driver(void);
#ifdef CONFIG_UNIX98_PTYS
extern struct tty_driver *ptm_driver;	/* Unix98 pty masters; for /dev/ptmx */
extern struct tty_driver *pts_driver;	/* Unix98 pty slaves;  for /dev/ptmx */
#endif

extern rwlock_t  tty_driver_guard;

#if defined(CONFIG_VE_NETDEV) || defined(CONFIG_VE_NETDEV_MODULE)
void ip_fragment_cleanup(struct ve_struct *envid);
void tcp_v4_kill_ve_sockets(struct ve_struct *envid);
struct fib_table * fib_hash_init(int id);
int move_addr_to_kernel(void *uaddr, int ulen, void *kaddr);
extern int main_loopback_init(struct net_device*);
int venet_init(void);
#endif

extern struct ve_struct *ve_list_head;
extern rwlock_t ve_list_guard;
extern struct ve_struct *get_ve_by_id(envid_t);
extern struct ve_struct *__find_ve_by_id(envid_t);

extern int do_setdevperms(envid_t veid, unsigned type,
		dev_t dev, unsigned mask);

#define VE_HOOK_INIT	0
#define VE_HOOK_FINI	1
#define VE_MAX_HOOKS	2

typedef int ve_hookfn(unsigned int hooknum, void *data);

struct ve_hook
{
	struct list_head list;
	ve_hookfn *hook;
	ve_hookfn *undo;
	struct module *owner;
	int hooknum;
	/* Functions are called in ascending priority. */
	int priority;
};

extern int ve_hook_register(struct ve_hook *vh);
extern void ve_hook_unregister(struct ve_hook *vh);

#endif
#endif
