/*
 *  include/linux/virtinfo.h
 *
 *  Copyright (C) 2005  SWsoft
 *  All rights reserved.
 *  
 *  Licensing governed by "linux/COPYING.SWsoft" file.
 *
 */

#ifndef __LINUX_VIRTINFO_H
#define __LINUX_VIRTINFO_H

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/page-flags.h>
#include <linux/notifier.h>

struct vnotifier_block
{
	int (*notifier_call)(struct vnotifier_block *self,
			unsigned long, void *, int);
	struct vnotifier_block *next;
	int priority;
};

extern struct semaphore virtinfo_sem;
void __virtinfo_notifier_register(int type, struct vnotifier_block *nb);
void virtinfo_notifier_register(int type, struct vnotifier_block *nb);
void virtinfo_notifier_unregister(int type, struct vnotifier_block *nb);
int virtinfo_notifier_call(int type, unsigned long n, void *data);

struct meminfo {
	struct sysinfo si;
	unsigned long active, inactive;
	unsigned long cache, swapcache;
	unsigned long committed_space, allowed;
	struct page_state ps;
	unsigned long vmalloc_total, vmalloc_used, vmalloc_largest;
};

#define VIRTINFO_DOFORK		0
#define VIRTINFO_DOEXIT		1
#define VIRTINFO_DOEXECVE	2
#define VIRTINFO_DOFORKRET	3
#define VIRTINFO_DOFORKPOST	4
#define VIRTINFO_EXIT		5
#define VIRTINFO_EXITMMAP	6
#define VIRTINFO_EXECMMAP	7
#define VIRTINFO_ENOUGHMEM	8
#define VIRTINFO_OUTOFMEM	9
#define VIRTINFO_PAGEIN		10
#define VIRTINFO_MEMINFO	11
#define VIRTINFO_SYSINFO	12
#define VIRTINFO_NEWUBC		13

enum virt_info_types {
	VITYPE_GENERAL,
	VITYPE_FAUDIT,
	VITYPE_QUOTA,
	VITYPE_SCP,

	VIRT_TYPES
};

#ifdef CONFIG_VZ_GENCALLS

static inline int virtinfo_gencall(unsigned long n, void *data)
{
	int r;

	r = virtinfo_notifier_call(VITYPE_GENERAL, n, data);
	if (r & NOTIFY_FAIL)
		return -ENOBUFS;
	if (r & NOTIFY_OK)
		return -ERESTARTNOINTR;
	return 0;
}

#else

#define virtinfo_gencall(n, data)	0

#endif

#endif /* __LINUX_VIRTINFO_H */
