#ifndef __LINUX_NET_AFUNIX_H
#define __LINUX_NET_AFUNIX_H
extern void unix_inflight(struct file *fp);
extern void unix_notinflight(struct file *fp);
extern void unix_gc(void);
extern void unix_destruct_fds(struct sk_buff *skb);
extern void wait_for_unix_gc(void);

#define UNIX_HASH_SIZE	256

extern struct hlist_head unix_socket_table[UNIX_HASH_SIZE + 1];
extern rwlock_t unix_table_lock;

extern atomic_t unix_tot_inflight;

static inline struct sock *first_unix_socket(int *i)
{
	struct sock *s;
	struct ve_struct *ve;

	ve = get_exec_env();
	for (*i = 0; *i <= UNIX_HASH_SIZE; (*i)++) {
		for (s = sk_head(&unix_socket_table[*i]);
		     s != NULL && !ve_accessible(VE_OWNER_SK(s), ve);
		     s = sk_next(s));
		if (s != NULL)
			return s;
	}
	return NULL;
}

static inline struct sock *next_unix_socket(int *i, struct sock *s)
{
	struct ve_struct *ve;

	ve = get_exec_env();
	for (s = sk_next(s); s != NULL; s = sk_next(s)) {
		if (!ve_accessible(VE_OWNER_SK(s), ve))
			continue;
		return s;
	}
	/* Look for next non-empty chain. */
	for ((*i)++; *i <= UNIX_HASH_SIZE; (*i)++) {
		for (s = sk_head(&unix_socket_table[*i]);
		     s != NULL && !ve_accessible(VE_OWNER_SK(s), ve);
		     s = sk_next(s));
		if (s != NULL)
			return s;
	}
	return NULL;
}

#define forall_unix_sockets(i, s) \
	for (s = first_unix_socket(&(i)); s; s = next_unix_socket(&(i),(s)))

struct unix_address {
	atomic_t	refcnt;
	int		len;
	unsigned	hash;
	struct sockaddr_un name[0];
};

struct unix_skb_parms {
	struct ucred		creds;		/* Skb credentials	*/
	struct scm_fp_list	*fp;		/* Passed files		*/
};

#define UNIXCB(skb) 	(*(struct unix_skb_parms*)&((skb)->cb))
#define UNIXCREDS(skb)	(&UNIXCB((skb)).creds)

#define unix_state_rlock(s)	read_lock(&unix_sk(s)->lock)
#define unix_state_runlock(s)	read_unlock(&unix_sk(s)->lock)
#define unix_state_wlock(s)	write_lock(&unix_sk(s)->lock)
#define unix_state_wunlock(s)	write_unlock(&unix_sk(s)->lock)

#ifdef __KERNEL__
/* The AF_UNIX socket */
struct unix_sock {
	/* WARNING: sk has to be the first member */
	struct sock		sk;
        struct unix_address     *addr;
        struct dentry		*dentry;
        struct vfsmount		*mnt;
        struct semaphore        readsem;
        struct sock		*peer;
        struct sock		*other;
        struct sock		*gc_tree;
        atomic_t                inflight;
        rwlock_t                lock;
        wait_queue_head_t       peer_wait;
#ifndef __GENKSYMS__
	unsigned int		gc_candidate : 1;
	unsigned int		gc_maybe_cycle : 1;
	unsigned char		recursion_level;
	struct list_head	link;
#endif
};
#define unix_sk(__sk) ((struct unix_sock *)__sk)
#endif
#endif
